<?php
namespace T3SBS\T3sbootstrap\Tasks;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;

class Scss extends \TYPO3\CMS\Scheduler\Task\AbstractTask {

	public function execute() {

		$this->configurationManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');
		$settings = $this->configurationManager->getConfiguration(
			\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS,
			't3sbootstrap',
			'm1'
		);

		if ( $settings['customScss'] ) {

			# Custom
			$customDir = $settings['customScssPath'] ? $settings['customScssPath'] : 'fileadmin/T3SB/SCSS/';
			$customPath = GeneralUtility::getFileAbsFileName($customDir);

			$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable('pages');
			$result = $queryBuilder
			   ->select('*')
			   ->from('pages')
			   ->where(
			      $queryBuilder->expr()->eq('is_siteroot', $queryBuilder->createNamedParameter(1, \PDO::PARAM_INT))
			   )
			   ->execute();
			$siteroots = $result->fetchAll();
			
			foreach ($siteroots as $key=>$siteroot) {

				if ($key === 0) {
					$customFileName = 'custom-variables.scss';
				} else {
					$customFileName = 'custom-variables-'.$siteroot['uid'].'.scss';
				}

				$customFile = $customPath.$customFileName;

				if (file_exists($customFile)) {
					$copyFile = $customPath.'_'.time().'-'.$customFileName;
					if (!copy($customFile, $copyFile)) {
						return FALSE;
					}
					self::deleteFilesFromDirectory($customPath);
				}

				$customFile = $customPath.$customFileName;

				if (!file_exists($customFile)) {

					if (!is_dir($customPath)) {
						mkdir($customPath, 0777, true);
					}

					$customContent = '// Variables overrides';

					if ( $settings['bootswatch'] ) {
						$customContent = GeneralUtility::getURL(GeneralUtility::getFileAbsFileName('EXT:t3sbootstrap/Resources/Public/Contrib/Bootswatch/'.strtolower($settings['bootswatch']).'.scss'));
						$customContent = str_replace(' !default', '', $customContent);
					}

					GeneralUtility::writeFile($customFile, $customContent);
				}

				# Include
				$includeDir = 'uploads/tx_t3sbootstrap/';
				$includePath = GeneralUtility::getFileAbsFileName($includeDir);

				if ($key === 0) {
					self::deleteFilesFromDirectory($includePath);
					$includeFileName = 'bootstrap.scss';
				} else {
					$includeFileName = 'bootstrap-'.$siteroot['uid'].'.scss';
				}

				$includeFile = $includePath.$includeFileName;

				if (!file_exists($includeFile)) {

					if (!is_dir($includePath)) {
						mkdir($includePath, 0777, true);
					}

					if ($key === 0) {
						$includeContent = '
@import "../../'.$customDir.'custom-variables";
@import "../../typo3conf/ext/t3sbootstrap/Resources/Public/Contrib/Bootstrap/scss/bootstrap";
						';
					} else {
						$includeContent = '
@import "../../'.$customDir.'custom-variables-'.$siteroot['uid'].'";
@import "../../typo3conf/ext/t3sbootstrap/Resources/Public/Contrib/Bootstrap/scss/bootstrap";
						';
					}
					GeneralUtility::writeFile($includeFile, $includeContent);
				}
			}

			# clean typo3temp/DynCss/
			$tempDir = 'typo3temp/DynCss/';
			$tempPath = GeneralUtility::getFileAbsFileName($tempDir);
			self::deleteFilesFromDirectory($tempPath);

			return TRUE;

		} else {

			return FALSE;

		}

	}

	public function deleteFilesFromDirectory($directory){
		if (is_dir($directory)) {
			if ($dh = opendir($directory)) {
				while (($file = readdir($dh)) !== false) {
					if ($file!='.' && $file !='..' && $file[0] != '_') {
						unlink(''.$directory.''.$file.'');
					}
				}
				closedir($dh);
			}
		}
	}

}
