<?php
/**
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Messaging\FlashMessage;
use TYPO3\CMS\Core\Messaging\FlashMessageService;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Update class for the extension manager.
 *
 * @package TYPO3
 * @subpackage tx_t3sbootstrap
 */
class ext_update
{

	/**
	 * Array of flash messages (params) array[][status,title,message]
	 *
	 * @var array
	 */
	protected $messageArray = [];

	/**
	 * @var \TYPO3\CMS\Core\Database\DatabaseConnection
	 */
	protected $databaseConnection;

	/**
	 * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManager
	 */
	protected $configurationManager;

	/**
	 * Constructor
	 */
	public function __construct()
	{
		$this->databaseConnection = $GLOBALS['TYPO3_DB'];
		$this->configurationManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Configuration\\ConfigurationManager');
	}

	/**
	 * Main update function called by the extension manager.
	 *
	 * @return string
	 */
	public function main()
	{
		$this->processUpdates();
		return $this->generateOutput();
	}

	/**
	 * Called by the extension manager to determine if the update menu entry
	 * should by showed.
	 *
	 * @return bool
	 */
	public function access()
	{
		return true;
	}

	/**
	 * The actual update function. Add your update task in here.
	 *
	 * @return void
	 */
	protected function processUpdates()
	{

		# Include
		$includeDir = 'uploads/tx_t3sbootstrap/migrate/';
		$includePath = GeneralUtility::getFileAbsFileName($includeDir);
		$includeFileName = 'constant.txt';
		$includeFile = $includePath.$includeFileName;
		$extbaseFrameworkSettings = $this->configurationManager->getConfiguration(
		\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FULL_TYPOSCRIPT);
		$old_constants = self::getOldConstants($extbaseFrameworkSettings['old_constants.']);

		if (!file_exists($includeFile) && ($old_constants['upgrade'] === 'TRUE') ) {
			$homepageUid = $old_constants['pages.']['homepage'];
			$homepage = $this->databaseConnection->exec_SELECTgetSingleRow('backend_layout', 'pages', 'uid='.$homepageUid);

			$onePageLayout = ($homepage['backend_layout'] == 'pagets__t3sbootstrap_10') ? 1 : 0;
			$jumbotron = self::isJumbotron($homepage['backend_layout']);

			$pages = $this->databaseConnection->exec_SELECTgetRows('*', 'pages', 'deleted=0');

			foreach ($pages as $page) {

				if ( $page['backend_layout'] ) {
					$backendLayout = $page['backend_layout'];
				} else {
					if ( $page['pid'] ) {
						if ( $pages[$page['pid']]['backend_layout'] ) {
							$backendLayout = $pages[$page['pid']]['backend_layout'];
						} else {
							if ( $pages[$page['pid']]['backend_layout_next_level'] ) {
								$backendLayout = $pages[$page['pid']]['backend_layout_next_level'];
							} else {
								$backendLayout = $pages[0]['backend_layout_next_level'];
							}
						}
					}
				}

				if ( !self::isJumbotron($backendLayout) ) {
					$jumbotronArr[$page['uid']] = $page['uid'];
				}

				if ( $page['backend_layout'] == 'pagets__t3sbootstrap_10' ) {
					$onePageLayoutArr[$page['uid']] = $page['uid'];
				}

			}

			$count_pagets_OneCol = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_1', 'pages')
				 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_4', 'pages')
				 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_6', 'pages')
				 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_10', 'pages'));
			if ( $count_pagets_OneCol ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_1', 'pages')
					 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_4', 'pages')
					 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_6', 'pages')
					 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_10', 'pages'),
					[
						'backend_layout' => 'pagets__OneCol'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout pagets__t3sbootstrap_1, pagets__t3sbootstrap_4, pagets__t3sbootstrap_6 and pagets__t3sbootstrap_10 to pagets__OneCol',
				$count_pagets_OneCol.' records have been updated!'];
			}

			$count_pagets_TwoCol_93 = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_2', 'pages')
				 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_7', 'pages'));
			if ( $count_pagets_TwoCol_93 ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_2', 'pages')
					 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_7', 'pages'),
					[
						'backend_layout' => 'pagets__TwoCol_9-3'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout pagets__t3sbootstrap_2 and pagets__t3sbootstrap_7 to pagets__TwoCol_9-3',
				$count_pagets_TwoCol_93.' records have been updated!'];
			}

			$count_pagets_ThreeCol = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_3', 'pages')
				 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_8', 'pages'));
			if ( $count_pagets_ThreeCol ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_3', 'pages')
					 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_8', 'pages'),
					[
						'backend_layout' => 'pagets__ThreeCol'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout pagets__t3sbootstrap_3 and pagets__t3sbootstrap_8 to pagets__ThreeCol',
				$count_pagets_ThreeCol.' records have been updated!'];
			}

			$count_pagets_TwoCol_39 = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_5', 'pages')
				 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_9', 'pages'));
			if ( $count_pagets_TwoCol_39 ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_5', 'pages')
					 . ' OR pages.backend_layout=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_9', 'pages'),
					[
						'backend_layout' => 'pagets__TwoCol_3-9'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout pagets__t3sbootstrap_5 and pagets__t3sbootstrap_9 to pagets__TwoCol_3-9',
				$count_pagets_TwoCol_39.' records have been updated!'];
			}

			$count_pagets_OneCol_next = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_1', 'pages')
				 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_4', 'pages')
				 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_6', 'pages')
				 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_10', 'pages'));
			if ( $count_pagets_OneCol_next ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_1', 'pages')
					 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_4', 'pages')
					 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_6', 'pages')
					 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_10', 'pages'),
					[
						'backend_layout_next_level' => 'pagets__OneCol'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout_next_level pagets__t3sbootstrap_1, pagets__t3sbootstrap_4, pagets__t3sbootstrap_6 and pagets__t3sbootstrap_10 to pagets__OneCol',
				$count_pagets_OneCol_next.' records have been updated!'];
			}

			$count_pagets_TwoCol_93_next = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_2', 'pages')
				 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_7', 'pages'));
			if ( $count_pagets_TwoCol_93_next ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_2', 'pages')
					 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_7', 'pages'),
					[
						'backend_layout_next_level' => 'pagets__TwoCol_9-3'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout_next_level pagets__t3sbootstrap_2 and pagets__t3sbootstrap_7 to pagets__TwoCol_9-3',
				$count_pagets_TwoCol_93_next.' records have been updated!'];
			}

			$count_pagets_ThreeCol_next = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_3', 'pages')
				 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_8', 'pages'));
			if ( $count_pagets_ThreeCol_next ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_3', 'pages')
					 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_8', 'pages'),
					[
						'backend_layout_next_level' => 'pagets__ThreeCol'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout_next_level pagets__t3sbootstrap_3 and pagets__t3sbootstrap_8 to pagets__ThreeCol',
				$count_pagets_ThreeCol_next.' records have been updated!'];
			}

			$count_pagets_TwoCol_39_next = $this->databaseConnection->exec_SELECTcountRows('*', 'pages',
				'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_5', 'pages')
				 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_9', 'pages'));
			if ( $count_pagets_TwoCol_39_next ) {
				$this->databaseConnection->exec_UPDATEquery(
					'pages',
					'pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_5', 'pages')
					 . ' OR pages.backend_layout_next_level=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_9', 'pages'),
					[
						'backend_layout_next_level' => 'pagets__TwoCol_3-9'
					]
				);
				$this->messageArray[] = [FlashMessage::OK,
				'Update backend_layout_next_level pagets__t3sbootstrap_5 and pagets__t3sbootstrap_9 to pagets__TwoCol_3-9',
				$count_pagets_TwoCol_39_next.' records have been updated!'];
			}

			$this->databaseConnection->exec_UPDATEquery(
				'pages',
				'pages.tx_t3sbootstrap_container=' . $this->databaseConnection->fullQuoteStr('', 'pages')
				 . ' AND pages.backend_layout!=' . $this->databaseConnection->fullQuoteStr('pagets__t3sbootstrap_10', 'pages'),
				[
					'tx_t3sbootstrap_container' => $old_constants['styles.']['container-fluid'] ? 'container-fluid': 'container'
				]
			);
			$this->databaseConnection->exec_UPDATEquery(
				'pages',
				'pages.tx_t3sbootstrap_smallColumns=' . $this->databaseConnection->fullQuoteStr('', 'pages'),
				[
					'tx_t3sbootstrap_smallColumns' => '3'
				]
			);


			if (!empty($jumbotronArr)) {
				$includeContent = '
[globalVar = TSFE:id = '.implode('|', $jumbotronArr).']
bootstrap.config.jumbotron_enable = 0
bootstrap.config.page_title = breadcrumb
[global]
			';
			}

			if (!empty($onePageLayoutArr)) {
				$includeContent .= '
[globalVar = TSFE:id = '.implode('|', $onePageLayoutArr).']
bootstrap.config.navbar_sectionmenu = 1
[global]
			';
			}

			if ($includeContent) {

				if (!is_dir($includePath)) {
					mkdir($includePath, 0777, true);
				}

				GeneralUtility::writeFile($includeFile, $includeContent);
				$this->messageArray[] = [FlashMessage::INFO, 'Enable configOverride in the EM config and add the following into your constants:', $includeContent];
			}

			$count_thumbnails = $this->databaseConnection->exec_SELECTcountRows('*', 'tt_content','CType = "t3sbs_thumbnails"');
			if ($count_thumbnails) {
				$this->databaseConnection->exec_UPDATEquery(
					'tt_content',
					'tt_content.CType=' . $this->databaseConnection->fullQuoteStr('t3sbs_thumbnails', 'tt_content'),
					[
						'CType' =>  't3sbs_card'
					]
				);
				$this->messageArray[] = [FlashMessage::OK, 'Migrate CType t3sbs_thumbnails to t3sbs_card', $count_thumbnails . ' records have been updated!'];
			}

			$count_thumbnails_container = $this->databaseConnection->exec_SELECTcountRows('*',
			'tt_content','CType = "gridelements_pi1" AND tx_gridelements_backend_layout = "thumbnails_container"');
			if ($count_thumbnails_container) {
				$this->databaseConnection->exec_UPDATEquery(
					'tt_content',
					'tt_content.CType=' . $this->databaseConnection->fullQuoteStr('gridelements_pi1', 'tt_content')
					.' AND tt_content.tx_gridelements_backend_layout=' . $this->databaseConnection->fullQuoteStr('thumbnails_container', 'tt_content'),
					[
						'tx_gridelements_backend_layout' =>  'card_wrapper'
					]
				);
				$this->messageArray[] = [FlashMessage::OK, 'Migrate thumbnails container to card wrapper', $count_thumbnails_container . ' records have been updated!'];
			}

			$count_panel = $this->databaseConnection->exec_SELECTcountRows('*', 'tt_content','CType = "t3sbs_panel"');
			if ($count_panel) {
				$this->databaseConnection->exec_UPDATEquery(
					'tt_content',
					'tt_content.CType=' . $this->databaseConnection->fullQuoteStr('t3sbs_panel', 'tt_content'),
					[
						'CType' =>  't3sbs_card'
					]
				);
				$this->messageArray[] = [FlashMessage::OK, 'Migrate CType t3sbs_panel to t3sbs_card', $count_panel . ' records have been updated!'];
			}

			$cards = $this->databaseConnection->exec_SELECTgetRows('*', 'tt_content','CType="t3sbs_card"');
			foreach ( $cards as $card ) {
				$this->databaseConnection->exec_UPDATEquery(
					'tt_content',
					'tt_content.uid=' . $this->databaseConnection->fullQuoteStr($card['uid'], 'tt_content'),
					[
						'pi_flexform' => self::getCardFlexform($card['bodytext'])
					]
				);
			}

			$cards_wrapper = $this->databaseConnection->exec_SELECTgetRows('*',
			'tt_content','tt_content.CType=' . $this->databaseConnection->fullQuoteStr('gridelements_pi1', 'tt_content')
			.' AND tt_content.tx_gridelements_backend_layout=' . $this->databaseConnection->fullQuoteStr('card_wrapper', 'tt_content'));
			foreach( $cards_wrapper as $card_wrapper ) {
				$this->databaseConnection->exec_UPDATEquery(
					'tt_content',
					'tt_content.uid=' . $this->databaseConnection->fullQuoteStr($card_wrapper['uid'], 'tt_content'),
					[
						'tx_t3sbootstrap_flexform' => self::getCardWrapperFlexform()
					]
				);
			}

			# affix
			$this->databaseConnection->exec_UPDATEquery(
				'tt_content',
				'tt_content.CType="menu_9"',
				[
					'CType' => 'menu_section'
				]
			);

			$this->databaseConnection->exec_UPDATEquery(
				'tt_content',
				'tt_content.header_layout=6',
				[
					'header_layout' => 2,
					'tx_t3sbootstrap_header_display' => 'display-4'
				]
			);

			$this->databaseConnection->exec_UPDATEquery(
				'tt_content',
				'tt_content.header_layout=7',
				[
					'header_layout' => 5
				]
			);

			if ($old_constants['navigation.']['navbar.']['enable']) {
				$navbarEnable = $old_constants['styles.']['navbar.']['style.']['inverse'] ? 'dark' : 'light';
			} else {
				$navbarEnable = '';
			}

			$this->databaseConnection->exec_UPDATEquery(
				'tx_t3sbootstrap_domain_model_config',
				'pid='.$homepageUid,
				[
					'company' => $old_constants['general.']['company'],
					'homepage_uid' => $homepageUid,
					'page_titlealign' => '',
					'footer_pid' => $old_constants['pages.']['footer.']['content.']['pid'],
					'navbar_enable' => $navbarEnable,
					'navbar_color' => $old_constants['styles.']['navbar.']['style.']['inverse'] ? 'dark' : 'light',
					'navbar_placement' => $old_constants['styles.']['navbar.']['style.']['fixed.']['top'] ? 'fixed-top' : '',
					'navbar_entrylevel' => $old_constants['navigation.']['navbar.']['entryLevel'],
					'navbar_excludeuiduist' => $old_constants['navigation.']['navbar.']['excludeUidList'],
					'navbar_hover' => $old_constants['navigation.']['navbar.']['hover'],
					'navbar_sectionmenu' => $onePageLayout,
					'navbar_height' => '56',
					'jumbotron_enable' => $jumbotron,
					'navbar_searchbox' => $old_constants['navigation.']['navbar.']['search.']['enable'],
					'navbar_lang_uid' => $old_constants['navigation_lang.']['languages'],
					'sidebar_enable' => $old_constants['navigation.']['sidebar-subMenu.']['enable'] ? 'Sub' : '',
					'sidebar_excludeuiduist' => $old_constants['navigation.']['sidebar-subMenu.']['excludeUidList'],
					'breadcrumb_enable' => $old_constants['navigation.']['breadcrumb.']['enable'],
					'meta_enable' => $old_constants['navigation.']['navbar-right.']['uidList'] ? 'end' : '',
					'meta_value' => $old_constants['navigation.']['navbar-right.']['uidList'] ? $old_constants['navigation.']['navbar-right.']['uidList'] : '',
					'meta_container' => 'container',
				]
			);

		} else {
			if ( !file_exists($includeFile) ) {
				$this->messageArray[] = [FlashMessage::WARNING, 'If you really want to upgrade, then you must set the following constant:',
				 'plugin.t3sbootstrap_configuration.upgrade = TRUE'];
			}
		}

		// Check for errors
		if ($this->databaseConnection->sql_error()) {
			$this->messageArray[] = [FlashMessage::ERROR, 'SQL-ERROR:', htmlspecialchars($this->databaseConnection->sql_error())];
			return $this->generateOutput();
		}
	}


	/**
	 * Generates output by using flash messages
	 *
	 * @return string
	 */
	protected function generateOutput()
	{
		 /** @var FlashMessageService $flashMessageService */
		$flashMessageService = GeneralUtility::makeInstance(FlashMessageService::class);
		$defaultFlashMessageQueue = $flashMessageService->getMessageQueueByIdentifier();

		if ($this->messageArray[0]) {
			foreach ($this->messageArray as $messageItem) {
				/** @var \TYPO3\CMS\Core\Messaging\FlashMessage $flashMessage */
				$flashMessage = GeneralUtility::makeInstance(
					FlashMessage::class,
					$messageItem[2],
					$messageItem[1],
					$messageItem[0]);
					$defaultFlashMessageQueue->enqueue($flashMessage);
			}
		} else {

			/** @var \TYPO3\CMS\Core\Messaging\FlashMessage $flashMessage */
			$flashMessage = GeneralUtility::makeInstance(
					FlashMessage::class,
				'Nothing to do - you\'re up to date!',
				'Update script',
				FlashMessage::INFO);
				$defaultFlashMessageQueue->enqueue($flashMessage);
		}

		return $defaultFlashMessageQueue->renderFlashMessages();
	}



	/**
	 * Get the old constants from v3 and set defaults
	 *
	 * @return array
	 */
	protected function getOldConstants($old_constants)
	{
		$old_constants['general.']['company'] = ($old_constants['general.']['company'] == '{$plugin.t3sbootstrap_configuration.general.company.name}')
		 ? '' : $old_constants['general.']['company'];
		$old_constants['pages.']['sitemap'] = ($old_constants['pages.']['sitemap'] == '{$plugin.t3sbootstrap_configuration.pages.sitemap.uidList}')
		 ? '' : $old_constants['pages.']['sitemap'];
		$old_constants['pages.']['homepage'] = ($old_constants['pages.']['homepage'] == '{$plugin.t3sbootstrap_configuration.pages.homepage.uid}')
		 ? 1 : $old_constants['pages.']['homepage'];
		$old_constants['navigation.']['navbar.']['enable'] =
		 ($old_constants['navigation.']['navbar.']['enable'] == '{$plugin.t3sbootstrap_configuration.navigation.navbar.enable}')
		 ? 1 : $old_constants['navigation.']['navbar.']['enable'];
		$old_constants['navigation.']['navbar.']['entryLevel'] = ($old_constants['navigation.']['navbar.']['entryLevel'] == '{$plugin.t3sbootstrap_configuration.navigation.navbar.entryLevel}')
		 ? 0 : $old_constants['navigation.']['navbar.']['entryLevel'];
		$old_constants['navigation.']['navbar.']['excludeUidList'] = ($old_constants['navigation.']['navbar.']['excludeUidList'] == '{$plugin.t3sbootstrap_configuration.navigation.navbar.excludeUidList}')
		 ? '' : $old_constants['navigation.']['navbar.']['excludeUidList'];
		$old_constants['navigation.']['navbar.']['hover'] = ($old_constants['navigation.']['navbar.']['hover'] == '{$plugin.t3sbootstrap_configuration.navigation.navbar.hover}')
		 ? 0 : $old_constants['navigation.']['navbar.']['hover'];
		$old_constants['navigation.']['breadcrumb.']['enable'] = ($old_constants['navigation.']['breadcrumb.']['enable'] == '{$plugin.t3sbootstrap_configuration.navigation.breadcrumb.enable}')
		 ? 1 : $old_constants['navigation.']['breadcrumb.']['enable'];
		$old_constants['navigation.']['sidebar-subMenu.']['enable'] = ($old_constants['navigation.']['sidebar-subMenu.']['enable'] == '{$plugin.t3sbootstrap_configuration.navigation.sidebar-subMenu.enable}')
		 ? 1 : $old_constants['navigation.']['sidebar-subMenu.']['enable'];
		$old_constants['navigation.']['sidebar-subMenu.']['excludeUidList'] = ($old_constants['navigation.']['sidebar-subMenu.']['excludeUidList'] == '{$plugin.t3sbootstrap_configuration.navigation.sidebar-subMenu.excludeUidList}')
		 ? '' : $old_constants['navigation.']['sidebar-subMenu.']['excludeUidList'];
		$old_constants['navigation.']['navbar.']['search.']['enable'] = ($old_constants['navigation.']['navbar.']['search.']['enable'] == '{$plugin.t3sbootstrap_configuration.navigation.navbar.search.enable}')
		 ? 0 : $old_constants['navigation.']['navbar.']['search.']['enable'];
		$old_constants['navigation_lang.']['enable'] = ($old_constants['navigation_lang.']['enable'] == '{$plugin.t3sbootstrap_configuration.navigation_lang.enable}')
		 ? 0 : $old_constants['navigation_lang.']['enable'];
		$old_constants['navigation_lang.']['languages'] = ($old_constants['navigation_lang.']['languages'] == '{$plugin.t3sbootstrap_configuration.navigation_lang.languages}')
		 ? '0,1' : $old_constants['navigation_lang.']['languages'];
		$old_constants['styles.']['container-fluid'] = ($old_constants['styles.']['container-fluid'] == '{$plugin.t3sbootstrap_configuration.styles.container-fluid}')
		 ? 0 : $old_constants['styles.']['container-fluid'];
		$old_constants['styles.']['jumbotron.']['fullwidth'] = ($old_constants['styles.']['jumbotron.']['fullwidth'] == '{$plugin.t3sbootstrap_configuration.styles.jumbotron.fullwidth}')
		 ? 1 : $old_constants['styles.']['jumbotron.']['fullwidth'];
		$old_constants['styles.']['jumbotron.']['bgImg.']['enable'] = ($old_constants['styles.']['jumbotron.']['bgImg.']['enable'] == '{$plugin.t3sbootstrap_configuration.styles.jumbotron.bgImg.enable}')
		 ? 0 : $old_constants['styles.']['jumbotron.']['bgImg.']['enable'];
		$old_constants['styles.']['jumbotron.']['bgImg.']['maxWidth'] = ($old_constants['styles.']['jumbotron.']['bgImg.']['maxWidth'] == '{$plugin.t3sbootstrap_configuration.styles.jumbotron.bgImg.maxWidth}')
		 ? 1920 : $old_constants['styles.']['jumbotron.']['bgImg.']['maxWidth'];
		$old_constants['styles.']['jumbotron.']['bgImg.']['maxHeight'] = ($old_constants['styles.']['jumbotron.']['bgImg.']['maxHeight'] == '{$plugin.t3sbootstrap_configuration.styles.jumbotron.bgImg.maxHeight}')
		 ? 400 : $old_constants['styles.']['jumbotron.']['bgImg.']['maxHeight'];
		$old_constants['styles.']['bsHeader.']['enable'] = ($old_constants['styles.']['bsHeader.']['enable'] == '{$plugin.t3sbootstrap_configuration.styles.bsHeader.enable}')
		 ? 1 : $old_constants['styles.']['bsHeader.']['enable'];
		$old_constants['styles.']['navbar.']['style.']['inverse'] = ($old_constants['styles.']['navbar.']['style.']['inverse'] == '{$plugin.t3sbootstrap_configuration.styles.navbar.style.inverse}')
		 ? 0 : $old_constants['styles.']['navbar.']['style.']['inverse'];
		$old_constants['styles.']['navbar.']['style.']['fixed.']['top'] = ($old_constants['styles.']['navbar.']['style.']['fixed.']['top'] == '{$plugin.t3sbootstrap_configuration.styles.navbar.style.fixed.top}')
		 ? 1 : $old_constants['styles.']['navbar.']['style.']['fixed.']['top'];
		$old_constants['styles.']['navbar.']['style.']['navbar.']['onTop'] = ($old_constants['styles.']['navbar.']['style.']['navbar.']['onTop'] == '{$plugin.t3sbootstrap_configuration.styles.navbar.onTop}')
		 ? 1 : $old_constants['styles.']['navbar.']['style.']['navbar.']['onTop'];
		$old_constants['styles.']['navbar.']['style.']['height'] = ($old_constants['styles.']['navbar.']['style.']['height'] == '{$plugin.t3sbootstrap_configuration.styles.navbar.style.height}')
		 ? 50 : $old_constants['styles.']['navbar.']['style.']['height'];
		$old_constants['styles.']['footer.']['enable'] = ($old_constants['styles.']['footer.']['enable'] == '{$plugin.t3sbootstrap_configuration.styles.footer.enable}')
		 ? 1 : $old_constants['styles.']['footer.']['enable'];
		$old_constants['styles.']['navbar.']['footer.']['fullwidth'] = ($old_constants['styles.']['navbar.']['footer.']['fullwidth'] == '{$plugin.t3sbootstrap_configuration.styles.navbar.footer.fullwidth}')
		 ? 1 : $old_constants['styles.']['navbar.']['footer.']['fullwidth'];
		$old_constants['styles.']['footer.']['sticky'] = ($old_constants['styles.']['footer.']['sticky'] == '{$plugin.t3sbootstrap_configuration.styles.footer.sticky}')
		 ? 1 : $old_constants['styles.']['footer.']['sticky'];
		$old_constants['extensions.']['indexed_search.']['page'] = ($old_constants['extensions.']['indexed_search.']['page'] == '{$plugin.t3sbootstrap_configuration.extensions.indexed_search.page}')
		 ? 1 : $old_constants['extensions.']['indexed_search.']['page'];

		return $old_constants;
	}


	/**
	 * Is Jumbotron
	 *
	 * @return integer
	 */
	protected function isJumbotron($backendLayout)
	{

		switch ($backendLayout) {
				 case 'pagets__t3sbootstrap_1':
				$jumbotron = 0;
				  break;
				 case 'pagets__t3sbootstrap_2':
				$jumbotron = 0;
				  break;
				 case 'pagets__t3sbootstrap_3':
				$jumbotron = 0;
				  break;
				 case 'pagets__t3sbootstrap_4':
				$jumbotron = 1;
				  break;
				 case 'pagets__t3sbootstrap_5':
				$jumbotron = 0;
				  break;
				 case 'pagets__t3sbootstrap_6':
				$jumbotron = 1;
				  break;
				 case 'pagets__t3sbootstrap_7':
				$jumbotron = 1;
				  break;
				 case 'pagets__t3sbootstrap_8':
				$jumbotron = 1;
				  break;
				 case 'pagets__t3sbootstrap_9':
				$jumbotron = 1;
				  break;
				 case 'pagets__t3sbootstrap_10':
				$jumbotron = 0;
				  break;
				 default:
				 $jumbotron = 1;
		}

		return $jumbotron;
	}


	/**
	 * Returns the card flexform.
	 *
	 * @return string
	 */
	protected function getCardFlexform($bodytext)
	{
		return '<?xml version="1.0" encoding="utf-8" standalone="yes" ?>
<T3FlexForms>
	<data>
		<sheet index="sDEF">
			<language index="lDEF">
				<field index="header.text">
					<value index="vDEF"></value>
				</field>
				<field index="text.top">
					<value index="vDEF">'
					. $bodytext .
					'</value>
				</field>
				<field index="text.bottom">
					<value index="vDEF"></value>
				</field>
				<field index="footer.text">
					<value index="vDEF"></value>
				</field>
			</language>
		</sheet>
	</data>
</T3FlexForms>';
	}


	/**
	 * Returns the card wrapper flexform.
	 *
	 * @return string
	 */
	protected function getCardWrapperFlexform()
	{
		return '<?xml version="1.0" encoding="utf-8" standalone="yes" ?>
<T3FlexForms>
	<data>
		<sheet index="sDEF">
			<language index="lDEF">
				<field index="card_wrapper">
					<value index="vDEF">deck</value>
				</field>
			</language>
		</sheet>
	</data>
</T3FlexForms>';
	}


}
